; ------------------------------------------------------------------------------------------------
; Rand - Random seed tests in asm - compiled with MASM
; ------------------------------------------------------------------------------------------------
; Testing 3 methods to random a float in the range [-1,1[ :
;
; USE_IQ_RAND is based on iq's work http://iquilezles.org/www/articles/sfrand/sfrand.htm
; USE_C_RAND is using the libc
; USE_DIV_RAND is using 
;
; author: alx@frequency.fr
; ------------------------------------------------------------------------------------------------
.686p
.model flat

; Uncomment one of this define :

USE_IQ_RAND = 1
;USE_DIV_RAND = 1
; USE_C_RAND = 1
; USE_EMPTY_RAND = 1

; ------------------------------------------------------------------------------------------------
.data
; ------------------------------------------------------------------------------------------------
RandSeed dd 1

ifdef USE_DIV_RAND
RandDiv dd 080000000h
endif

ifdef USE_C_RAND
; External C rand function
_imp__rand proto c
RandMax dw 32768
endif

; ------------------------------------------------------------------------------------------------
.code
; ------------------------------------------------------------------------------------------------
; Rand 
; Empty reference
; ------------------------------------------------------------------------------------------------
ifdef USE_EMPTY_RAND
_Rand proc
	fldz
	ret										;#1 return to caller
_Rand endp									;#34 total bytes
endif

; ------------------------------------------------------------------------------------------------
; Rand 
; USE_IQ_RAND is based on iq's work http://iquilezles.org/www/articles/sfrand/sfrand.htm
; ------------------------------------------------------------------------------------------------
ifdef USE_IQ_RAND
_Rand proc
	imul eax, dword ptr [RandSeed], 16807	;#A eax = RandSeed * 16807
	mov dword ptr [RandSeed], eax			;#5 RandSeed = eax
	mov al, 080h							;#2 eax: rrrrrr80h where r are bits from random seeds
	ror eax, 8								;#3 eax: 80rrrrrrh. value = s * 2^(e-127). e is set to 128, thus 040h value ( sEee eeee efff ffff ffff ffff ffff ffff)
	shr eax, 1								;#2 eax: 40rrrrrrh
	push eax								;#1 push eax on stack to load it as a float. rand [2,4)
	fld dword ptr [esp]						;#3 load rand st0 = rand [2,4)
	push 3									;#2 load 3
	fisub dword ptr [esp]					;#3 st(0) = rand [2,4) - 3 = rand[-1, 1)
	pop eax									;#1 free stack
	pop eax									;#1 free stack	
	ret										;#1 return to caller
_Rand endp									;#34 total bytes
endif

; ------------------------------------------------------------------------------------------------
; Rand 
; USE_DIV_RAND is using p
; ------------------------------------------------------------------------------------------------
ifdef USE_DIV_RAND
_Rand proc
	imul eax, dword ptr [RandSeed], 16807	;#A eax = RandSeed * 16807
	mov dword ptr [RandSeed], eax			;#5 RandSeed = eax
	fild dword ptr [RandSeed]				;#6 load RandSeed as an integer
	fidiv dword ptr [RandDiv]				;#6 div by max int value (absolute) = eax / (-2^31)
	ret										;#1 return to caller
_Rand endp									;#28 total bytes
endif

; ------------------------------------------------------------------------------------------------
; Rand 
; USE_C_RAND is using the rand() libc function
; ------------------------------------------------------------------------------------------------
ifdef USE_C_RAND
_Rand proc
	call dword ptr [_imp__rand]				;#6 Call c Rand
	push eax								;#1 st(0)		st(1)
	fild dword ptr [esp]					;#3 rand		  
	fidiv word ptr [RandMax]				;#6 (-1,0]		  
	fchs									;#2 [0,1)
	fadd st(0),st(0)						;#2 [0,2)		  
	fld1									;#2	1			[0,2)
	fsubp									;#2 [-1,+1)
	pop eax									;#1 clean stack 
	ret										;#1
_Rand endp									;#26 total bytes
endif
      
end